@@ -149,6 +149,72 @@ sub Fields {
     return @{ $cache->{ lc $table } || [] };
 }
 
+=head2 SimpleDateTimeFunctions
+
+Returns hash reference with specific date time functions of this
+database for L<DBIx::SearchBuilder::Handle/DateTimeFunction>.
+
+=cut
+
+sub SimpleDateTimeFunctions {
+    my $self = shift;
+    return $self->{'_simple_date_time_functions'} ||= {
+        %{ $self->SUPER::SimpleDateTimeFunctions(@_) },
+        datetime   => 'datetime(?)',
+        time       => 'time(?)',
+
+        hourly     => "strftime('%Y-%m-%d %H', ?)",
+        hour       => "strftime('%H', ?)",
+
+        date       => 'date(?)',
+        daily      => 'date(?)',
+
+        day        => "strftime('%d', ?)",
+        dayofmonth => "strftime('%d', ?)",
+
+        monthly    => "strftime('%Y-%m', ?)",
+        month      => "strftime('%m', ?)",
+
+        annually   => "strftime('%Y', ?)",
+        year       => "strftime('%Y', ?)",
+
+        dayofweek  => "strftime('%w', ?)",
+        dayofyear  => "strftime('%j', ?)",
+        weekofyear => "strftime('%W', ?)",
+    };
+}
+
+sub ConvertTimezoneFunction {
+    my $self = shift;
+    my %args = (
+        From  => 'UTC',
+        To    => undef,
+        Field => '',
+        @_
+    );
+    return $args{'Field'} unless $args{From} && $args{'To'};
+    return $args{'Field'} if lc $args{From} eq lc $args{'To'};
+
+    my $res;
+    if ( lc($args{'To'}||'') eq 'utc' ) {
+        $res = "datetime($args{'Field'}, 'utc')";
+    }
+    elsif ( lc($args{'From'}||'') eq 'utc' ) {
+        $res = "datetime($args{'Field'}, 'localtime')";
+    }
+    else {
+        warn "SQLite only supports TZ convesion from UTC or to UTC";
+        $res = $args{'Field'};
+    }
+    return $res;
+}
+
+sub _DateTimeIntervalFunction {
+    my $self = shift;
+    my %args = ( From => undef, To => undef, @_ );
+
+    return "strftime('%s',$args{'To'}) - strftime('%s',$args{'From'})";
+}
 
 1;
 
